from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = """
      lookup: morph_cypher
        author: Adam Hicks <ahicks@morpheusdata.com>
        version_added: "0.1.0"
        short_description: read secrets from Morpheus's Cypher Vault
        description:
            - read secrets from Morpheus's Cypher Vault
        options:
          baseurl:
            description: URL for Morpheus
            required: True
          authtype:
            description: userpass based auth or token based auth
            required: True
          secret_key:
            description: The secret key to be read from Morpheus
            required: True
          username:
            description: Username for authenticating against Morpheus
            required: False
          password:
            description: password for Username for authenticating against Morpheus
            required: False
          api_token:
            description: Morpheus API Token for authenticating against Morpheus
            required: False
          ssl_verify:
            description: Ignore SSL warnings True or False
            required: False
        notes:
          - if read in variable context, the file can be interpreted as YAML if the content is valid to the parser.
          - this lookup does not understand globing --- use the fileglob lookup instead.
"""

EXAMPLES = """
- debug:
    msg: "{{ lookup('morph_cypher', 'baseurl=https://sandbox.morpheusdata.com authtype=token api_token=xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx secret_key=password/spark')}}"

- debug:
    msg: "{{ lookup('morph_cypher', 'baseurl=https://sandbox.morpheusdata.com authtype=userpass username=slim_shady password=password secret_key=secret/hello') }}"

- debug:
    msg: "{{ lookup('morph_cypher', 'baseurl=https://sandbox.morpheusdata.com authtype=token api_token=xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx ssl_verify=False secret_key=key/256/myKey') }}"
"""

RETURN = """
_raw:
  description:
    - secrets(s) requested
"""

import json
import os
import requests
import sys
import warnings
import posixpath

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

from ansible.errors import AnsibleError
from ansible.plugins.lookup import LookupBase
from ansible.module_utils.morpheus import (
    morph_auth,
    morph_get_client,
    morph_post_client,
    morphtoken
)
from ansible.utils.display import Display
display = Display()


class LookupModule(LookupBase):

    def _get_params(self, params):
        authtype = params['authtype']
        if authtype == 'userpass':
            if 'username' not in params:
                params['username'] = os.environ.get('MORPH_USER')
            if 'password' not in params:
                params['password'] = os.environ.get('MORPH_PASSWORD')
            params['api_token'] = morph_auth(params)
        else:
            if 'api_token' not in params:
                params['api_token'] = morphtoken()
        if 'baseurl' not in params:
            params['baseurl'] = os.environ.get('MORPH_ADDR')
        if 'ssl_verify' not in params:
            params['ssl_verify'] = os.environ.get('MORPH_SSL_VERIFY')
        if 'secret_key' not in params:
            params['secret_key'] = os.environ.get('MORPH_SECRET')
        return params


    def run(self, terms, variables=None, **kwargs):
        cypher_args = terms[0].split()
        params = {}
        ret = []

        for item in cypher_args:
            try:
                key, value = item.split('=')
            except ValueError:
                raise AnsibleError("morph_cypher lookup plugin needs key=value pairs, but received %s" %terms)
            params[key] = value
        
        query = {'itemKey': params['secret_key']}

        cypher = posixpath.join('api', 'cypher')
        client = morph_get_client(self._get_params(params), cypher, query)

        if len(client['cyphers']) == 0:
            new_query = {'cypher': query}
            new_client = morph_post_client(self._get_params(params), cypher, new_query)
            match = new_client['cypher']['id']
        else:
            match = [d['id'] for d in client['cyphers']][0]

        url = urljoin(params['baseurl'], cypher)
        headers = {'Authorization': 'BEARER ' + params['api_token']}
        new_cypher = posixpath.join('cypher', str(match), 'decrypt')
        secret_url = urljoin(url, new_cypher)
        new_resp = requests.get(secret_url, headers=headers, verify=params['ssl_verify'])
        ret.append(new_resp.json()['cypher']['itemValue'])
        return ret


#!/usr/bin/python
import requests
import json
import os
import posixpath
from ansible.module_utils.morpheus import (
    morph_get_client,
    morph_argspec,
    morph_init, 
    morphwrapper
)
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin
try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode


ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '0.0.2'}
DOCUMENTATION = '''
---
module: morph_cypher
version_added: "0.1.0"
short_description: Morpheus Cypher read module
description:
    - Module to read from Morpheus Cypher service.
options:
    baseurl:
        description:
            - url for Morpheus
        default:
            - to environment variable MORPH_ADDR
    authtype:
        description:
            - Defines how module builds client to Morpheus, through credentials or token
        default:
            - to environment variable MORPH_AUTHTYPE as token
    api_token:
        description:
            - API Token for Morpheus
        default:
            - to environment variable MORPH_TOKEN
    username:
        description:
            - Morpheus Username for userpass based auth
        default:
            - to environment variable MORPH_USER
    password:
        description:
            - Morpheus Password for userpass based auth
        default:
            - to environment variable MORPH_PASSWORD
    secret_key:
        description:
            - Secret key to read value from Morpheus
        default:
            - to environment variable MORPH_SECRET
    ssl_verify:
        description:
            - ignore ssl true or false
        default:
            - defaults to ssl_verify = True
    register:
        description:
            - variable to register result.
'''


@morphwrapper
def morph_secret(params):

    result = { "changed": False, "rc": 0}
    cypher = posixpath.join('api', 'cypher')
    query = {'itemKey': params['secret_key']}
    client = morph_get_client(params, cypher, query)
    #json_data = requests.get(url, headers=headers).json()
    
    if len(client['cyphers']) == 0:
        new_query = {'cypher': query}
        new_client = morph_post_client(self._get_params(params), cypher, new_query)
        match = new_client['cypher']['id']
    else:
        match = [d['id'] for d in client['cyphers']][0]

    url = urljoin(params["baseurl"], cypher)
    headers = {"Authorization": "BEARER " + params["api_token"]}
    new_cypher = posixpath.join('cypher', str(match), 'decrypt')
    secret_url = urljoin(url, new_cypher)
    new_resp = requests.get(secret_url, headers=headers, verify=params['ssl_verify'])
    #new_resp = morph_get_client(params, new_cypher)
    #result['secret'] = new_resp["cypher"]["itemValue"]
    result['secret'] = new_resp.json()["cypher"]["itemValue"]

    return result


def main():

    fields = morph_argspec()
    fields['secret_key'] = dict(required=True, default=os.environ.get('MORPH_SECRET'), type='str')

    module = morph_init(fields)
    
    result = morph_secret(module.params)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


if __name__=="__main__":
    main()


#!/usr/bin/python
import requests
import os
import json
from ansible.module_utils.basic import AnsibleModule
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin
try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode


def morph_argspec():
    argument_spec = dict(
        baseurl = dict(required=False, default=os.environ.get('MORPH_ADDR', ''), type='str'),
        ssl_verify = dict(required=False, default=os.environ.get('MORPH_SSL_VERIFY', True), type='bool'),
        authtype = dict(required=False, default=os.environ.get('MORPH_AUTHTYPE', 'token'), type='str'),
        api_token = dict(required=False, default=morphtoken(), type='str', no_log=True),
        username = dict(required=False, default=os.environ.get('MORPH_USER', ''), type='str'),
        password = dict(required=False, default=os.environ.get('MORPH_PASSWORD', ''), type='str', no_log=True)
    )
    return argument_spec


def morph_init(argument_spec, supports_check_mode=False):
    return AnsibleModule(argument_spec=argument_spec, supports_check_mode=supports_check_mode)


def morphtoken():
    if 'MORPH_TOKEN' in os.environ:
        return os.environ['MORPH_TOKEN']
    token_file = os.path.expanduser('~/.morphtoken')
    if os.path.exists(token_file):
        with open(token_file, 'r') as f:
            return f.read().strip() 
    return ''


def morph_get_client(params, endpoint, query=None):
    verify = params.get('ssl_verify')
    authtype = params.get('authtype')
    baseurl = params.get('baseurl')
    if authtype == 'userpass':
        token = morph_auth(params)
    else:
        token = params.get('api_token')
    url = urljoin(baseurl, endpoint)
    headers = {"Authorization": "BEARER " + token}
    json_data = requests.get(url, headers=headers, params=query, verify=verify).json()
    return json_data

def morph_post_client(params, endpoint, query=None):
    verify = params.get('ssl_verify')
    authtype = params.get('authtype')
    baseurl = params.get('baseurl')
    if authtype == 'userpass':
        token = morph_auth(params)
    else:
        token = params.get('api_token')
    url = urljoin(baseurl, endpoint)
    headers = {"Authorization": "BEARER " + token}
    headers["Content-Type"] = "application/json"
    json_data = requests.post(url, headers=headers, json=query, verify=verify).json()
    return json_data

    

def morph_auth(params):
    username = params.get('username')
    password = params.get('password')
    baseurl = params.get('baseurl')
    url = urljoin(baseurl, 'oauth')
    access = {
        "grant_type": "password",
        "scope": "write",
        "client_id": "morph-customer"
        }
    url2 = url + '/token?{}'.format(urlencode(access))
    payload = "username=" + username + "&password=" + password
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'User-agent': 'curl/7.51.0'
        }
    res = requests.post(url2, data=payload, headers=headers)
    json_response = json.loads(res.text)
    return json_response["access_token"]


def morphwrapper(function):
    def wrapper(*args, **kwargs):
        result = { "changed": False, "rc" : 0}
        try:
            result.update(function(*args, **kwargs))
        except Exception as e:
            result['rc'] = 1
            result['failed'] = True
            result['msg'] = u"Exception: " + str(e)
        return result
    return wrapper


#!/usr/bin/env python
from setuptools import setup

py_files = [
    "ansible/module_utils/morpheus",
    "ansible/plugins/lookup/morph_cypher"
]
files = [
    "ansible/modules/morpheus",
]

long_description=open('README.md', 'r').read()

setup(
    name='ansible-modules-morpheus',
    version='0.2.0',
    description='Ansible Modules for Morpheus Data',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Adam Hicks',
    author_email='ahicks@morpheusdata.com',
    url='https://github.com/gomorpheus/ansible-modules-morpheus',
    py_modules=py_files,
    packages=files,
    install_requires = [
        'ansible>=2.0.0',
        'requests>=2.21.0'
    ],
)
